import { Pipe, EventEmitter, Directive, Output, HostListener, Injectable, ɵɵdefineInjectable, Component, Input, NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { AngularFireStorage, AngularFireStorageModule } from '@angular/fire/storage';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const FILE_SIZE_UNITS = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
/** @type {?} */
const FILE_SIZE_UNITS_LONG = ['Bytes', 'Kilobytes', 'Megabytes', 'Gigabytes', 'Pettabytes', 'Exabytes', 'Zettabytes', 'Yottabytes'];
class FileSizePipe {
    /**
     * @param {?} sizeInBytes
     * @param {?=} longForm
     * @return {?}
     */
    transform(sizeInBytes, longForm) {
        /** @type {?} */
        const units = longForm
            ? FILE_SIZE_UNITS_LONG
            : FILE_SIZE_UNITS;
        /** @type {?} */
        let power = Math.round(Math.log(sizeInBytes) / Math.log(1024));
        power = Math.min(power, units.length - 1);
        /** @type {?} */
        const size = sizeInBytes / Math.pow(1024, power);
        // size in new units
        /** @type {?} */
        const formattedSize = Math.round(size * 100) / 100;
        // keep up to 2 decimals
        /** @type {?} */
        const unit = units[power];
        return size ? `${formattedSize} ${unit}` : '0';
    }
}
FileSizePipe.decorators = [
    { type: Pipe, args: [{
                name: 'fileSize'
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropZoneDirective {
    constructor() {
        this.dropped = new EventEmitter();
        this.hovered = new EventEmitter();
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onDrop($event) {
        $event.preventDefault();
        this.dropped.emit($event.dataTransfer.files);
        this.hovered.emit(false);
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onDragOver($event) {
        $event.preventDefault();
        this.hovered.emit(true);
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onDragLeave($event) {
        $event.preventDefault();
        this.hovered.emit(false);
    }
}
DropZoneDirective.decorators = [
    { type: Directive, args: [{
                selector: '[dropZone]'
            },] }
];
/** @nocollapse */
DropZoneDirective.ctorParameters = () => [];
DropZoneDirective.propDecorators = {
    dropped: [{ type: Output }],
    hovered: [{ type: Output }],
    onDrop: [{ type: HostListener, args: ['drop', ['$event'],] }],
    onDragOver: [{ type: HostListener, args: ['dragover', ['$event'],] }],
    onDragLeave: [{ type: HostListener, args: ['dragleave', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FileUploadFirestorageService {
    constructor() { }
}
FileUploadFirestorageService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
FileUploadFirestorageService.ctorParameters = () => [];
/** @nocollapse */ FileUploadFirestorageService.ngInjectableDef = ɵɵdefineInjectable({ factory: function FileUploadFirestorageService_Factory() { return new FileUploadFirestorageService(); }, token: FileUploadFirestorageService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FileUploadFirestorageComponent {
    /**
     * @param {?} storage
     */
    constructor(storage) {
        this.storage = storage;
        this.res = new EventEmitter();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggleHover(event) {
        this.isHovering = event;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    startUpload(event) {
        // The File object
        /** @type {?} */
        const file = event.item(0);
        // Client-side validation example
        if (this.onlyType) {
            if (!(this.onlyType.includes(file.type.split('/')[0]))) {
                console.error('unsupported file type: ', file.type.split('/')[0]);
                return;
            }
        }
        if (this.maxSize) {
            if (file.size >= this.maxSize) {
                console.error('unsupported size file. Its too heavy ', file.size);
                return;
            }
        }
        if (!this.onlyType && (file.type.split('/')[0] !== 'image' && file.type.split('/')[0] !== 'pdf')) {
            console.error('unsupported file type: ', file.type.split('/')[0]);
            return;
        }
        // The storage path
        /** @type {?} */
        const path = `${this.storagePath}/${new Date().getTime()}_${file.name}`;
        // // Totally optional metadata
        // const customMetadata = { app: 'Murbinas Painting Uploader' };
        // The main task
        this.task = this.storage.upload(path, file);
        // Progress monitoring
        this.percentage = this.task.percentageChanges();
        this.snapshot = this.task.snapshotChanges();
        // The file's download URL
        this.task.then((/**
         * @param {?} d
         * @return {?}
         */
        d => {
            d.ref.getDownloadURL().then((/**
             * @param {?} url
             * @return {?}
             */
            url => {
                this.snapshot = null;
                this.percentage = null;
                this.res.emit({ thumbnail: url, locationPath: d.ref.fullPath.split('/').pop() });
            }));
        }));
    }
    // Determines if the upload task is active
    /**
     * @param {?} snapshot
     * @return {?}
     */
    isActive(snapshot) {
        return snapshot.state === 'running' && snapshot.bytesTransferred < snapshot.totalBytes;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
FileUploadFirestorageComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-file-upload-firestorage',
                template: `
  <div class="dropzone" dropZone (hovered)="toggleHover($event)" (dropped)="startUpload($event)"
  [class.hovering]="isHovering">



  <h3> File Drop Zone </h3>
  <h6> Drop your file here </h6>

  <div class="file">
    <label class="file-label">
      <input class="file-input" type="file" (change)="startUpload($event.target.files)">
      <span class="file-cta">
        <span class="file-icon">
          <i class="fa fa-upload"></i>
        </span>
        <span class="file-label">
          or <strong>click here</strong> to choose a file…
        </span>
      </span>
    </label>
  </div>
</div>

<div *ngIf="(percentage | async) as pct">

  <progress class="progress is-info" [value]="pct || 0" max="100">
  </progress>

  {{ pct | number }}%

</div>

<div *ngIf="(snapshot | async) as snap">
  {{ snap.bytesTransferred | fileSize }} of {{ snap.totalBytes | fileSize }}

  <button (click)="task.pause()" class="button is-warning" [disabled]="!isActive(snap)">Pause</button>
  <button (click)="task.cancel()" class="button is-danger" [disabled]="!isActive(snap)">Cancel</button>
  <button (click)="task.resume()" class="button is-info" [disabled]="!(snap?.state === 'paused')">Resume</button>

</div>
  `,
                styles: [`
    .dropzone {
      display: flex;
      align-items: center;
      justify-content: center;
      flex-direction: column;
      height: 100%;
      min-height: 100px;
      border: 2px dashed #f16624;
      border-radius: 5px;
      background: white;
      margin: 0;
      cursor:pointer;
      padding: .5rem;
    }

    .dropzone:hover {
      border: 2px solid #f16624;
      color: #dadada !important;
    }

    .dropzone .file input.file-input{
        width: 0;
    }

    progress::-webkit-progress-value {
        transition: width 0.1s ease;
    }
  `]
            }] }
];
/** @nocollapse */
FileUploadFirestorageComponent.ctorParameters = () => [
    { type: AngularFireStorage }
];
FileUploadFirestorageComponent.propDecorators = {
    res: [{ type: Output }],
    storagePath: [{ type: Input }],
    maxSize: [{ type: Input }],
    onlyType: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FileUploadFirestorageModule {
}
FileUploadFirestorageModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    FileSizePipe,
                    DropZoneDirective,
                    FileUploadFirestorageComponent,
                ],
                imports: [
                    AngularFireStorageModule,
                    CommonModule
                ],
                exports: [
                    FileSizePipe,
                    DropZoneDirective,
                    FileUploadFirestorageComponent,
                ],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

export { DropZoneDirective, FileSizePipe, FileUploadFirestorageComponent, FileUploadFirestorageModule, FileUploadFirestorageService };
//# sourceMappingURL=file-upload-firestorage.js.map
